/*
 * HTTPcat, on GNU license.
 * Author: Andrzej Godziuk (GDR!)
 * http://www.GDR.w.pl/
 */


#include <winsock.h>
#include <winbase.h>
#include <stdio.h>
#include <time.h>

char *ip2str(long ip)
{
	static char b[17];

	sprintf(b, "%d.%d.%d.%d", ip>>24, (ip>>16)&0xff, (ip>>8)&0xff, ip&0xff);
	return b;
}

long getip(char *host)
{
    struct hostent *he;
	

    if ((he=gethostbyname(host)) == NULL) {  // get the host info 
        return 0;
    }

	return ntohl(*((long *)he->h_addr));
}

long getip_gui(char *host, HWND stat)
{
    struct hostent *he;
	

	SetWindowText(stat, "Resolving host name...");
    if ((he=gethostbyname(host)) == NULL) {  // get the host info 
		SetWindowText(stat, "Resolving host name failed.");
        return 0;
    }
	SetWindowText(stat, "Host name resolved");

	return ntohl(*((long *)he->h_addr));
}

int fsockopen(char *host, int port)
{
	int sockfd;
    struct hostent *he;
    struct sockaddr_in their_addr; 

        if ((he=gethostbyname(host)) == NULL) {  // get the host info 
#ifndef FINAL
            perror("gethostbyname");
#endif
            return 0;
        }

       if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
#ifndef FINAL
            perror("socket");
#endif
            return 0;
        }

        their_addr.sin_family = AF_INET;    // host byte order 
        their_addr.sin_port = htons(port);  // short, network byte order 
        their_addr.sin_addr = *((struct in_addr *)he->h_addr);
        memset(&(their_addr.sin_zero), '\0', 8);  // zero the rest of the struct 

        if (connect(sockfd, (struct sockaddr *)&their_addr,
                                              sizeof(struct sockaddr)) == -1) {
#ifndef FINAL
            perror("connect");
#endif
            return 0;
        }
		return sockfd;
}

int fsockopen_gui(char *host, int port, HWND stat)
{
	int sockfd;
    struct hostent *he;
    struct sockaddr_in their_addr; 
	char buf[1024];


	sprintf(buf, "Getting host name [%s]...", host);
	SetWindowText(stat, buf);

	if ((he=gethostbyname(host)) == NULL) {  // get the host info 
#ifndef FINAL
		sprintf(buf, "Error getting host name [%s]", host);
		SetWindowText(stat, buf);
#endif
            return 0;
        }

       if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
#ifndef FINAL
            perror("socket");
#endif
            return 0;
        }

        their_addr.sin_family = AF_INET;    // host byte order 
        their_addr.sin_port = htons(port);  // short, network byte order 
        their_addr.sin_addr = *((struct in_addr *)he->h_addr);
        memset(&(their_addr.sin_zero), '\0', 8);  // zero the rest of the struct 

	sprintf(buf, "Connecting to host [%s]...", host);
	SetWindowText(stat, buf);

        if (connect(sockfd, (struct sockaddr *)&their_addr,
                                              sizeof(struct sockaddr)) == -1) {
		sprintf(buf, "Connect call failed [%s]", host);
		SetWindowText(stat, buf);
            return 0;
        }
	sprintf(buf, "Connected to host [%s]", host);
	SetWindowText(stat, buf);
		return sockfd;
}

int sendall(int sockfd,const char *data, int len)
{
	int sent;

	do
	{	
		sent = send(sockfd,data,len,0);

		if (sent <= 0) {return FALSE;}

		len  -= sent;
		data += sent;
	} 
	while(len>0);

	return TRUE;

}

int sendall_gui(int sockfd,const char *data, int len, HWND stat)
{
	int sent;
	int initial=len;
	char buf[1024];

	do
	{	
		sprintf(buf, "Sent %d / %d bytes", initial-len, initial);
		SetWindowText(stat, buf);

		sent = send(sockfd,data,len,0);

		if (sent <= 0) {return FALSE;}

		len  -= sent;
		data += sent;
	} 
	while(len>0);

	sprintf(buf, "Sent %d / %d bytes (SEND completed)", initial, initial);
	SetWindowText(stat, buf);

	return TRUE;

}

int recvall_gui(int sockfd,char **data, int headeronly, HWND stat)
{
	int recvd=0;
	int s_recvd=0;
	int len=0;
	int stopped=0;
	int bufsize = 1024;
	int datatime=0;
	int hdsize=0, dtsize=0;
	char buf[1024];
	char recvbuf[1030];
	char *p;
	time_t tb;
	

	*data = malloc(1024);
	p = (char*)*data;

	tb = clock();
	do
	{	
		if(len)
		{
			int t = (clock()-tb)/CLOCKS_PER_SEC;
			if(t==0) t=1;
			sprintf(buf, "Received %d / %d bytes (%d %%) - avg. %d bytes/sec", s_recvd, len, (s_recvd*100)/len, s_recvd/t);
		}
		else
		{
			int t = (clock()-tb)/CLOCKS_PER_SEC;
			if(t==0) t=1;
			sprintf(buf, "Received %d bytes (Content-Length unknown) - avg. %d bytes/sec", s_recvd, s_recvd/t);
		}
		SetWindowText(stat, buf);

		recvd = recv(sockfd, recvbuf, 1024, 0);

		s_recvd += recvd;

		memcpy(p, recvbuf, recvd);


		p += recvd;
		realloc((char*)*data, s_recvd+2048);
		(*data)[s_recvd+1023]=0;

		if(!datatime)
		{
			char *q;

			q = strstr(*data, "Content-Length: ");
			if(q)
			{
				q += strlen("Content-Length: ");
				len = atoi(q);
			}
		}
		else
		{
			dtsize+=recvd;
		}

		if(!datatime)
		{
			char *r;
			if(r=strstr(*data, "\r\n\r\n") ) 
			{
				if(headeronly) {*(r+4)=0; stopped=1;}
				datatime=1;
				hdsize = (r-(*data))+4;
			}
			
			
		}
	} 
	while( (recvd>0) && (!stopped) );

	dtsize = s_recvd - hdsize;

	sprintf(buf, "RECV completed - got %d bytes (%d header, %d content) in %d seconds %s", s_recvd, hdsize, dtsize, (clock()-tb)/CLOCKS_PER_SEC, stopped?"(header only)":"");
	SetWindowText(stat, buf);

	return s_recvd;

}
